"""
Usage: converter.py <dataset dir> <save dir> <input width> <input height> <train set size>
    dataset dir: path to dataset root directory
    save dir: path to converted data output directory
    max image size (eg. 140 140)
    train subset relative size (eg. 0.6)
"""
import os
import sys
import random
from datasets.camvid import CamVid
import h5py
from neon.backends import gen_backend


def shuffle(arr):
    result = []
    for _ in range(len(arr)):
        source_index = random.randint(0, len(arr) - 1)
        result.append(arr.pop(source_index))
    return result

if (__name__ == '__main__'):
    if (len(sys.argv) != 6):
        print(__doc__)
        exit(0)

    dataset_root = os.path.abspath(sys.argv[1])
    save_dir = os.path.abspath(sys.argv[2])
    max_image_size = (int(sys.argv[3]), int(sys.argv[4]))
    train_set_size = float(sys.argv[5])
    if ((train_set_size <= 0) or (1 < train_set_size)):
        print("Expected train subset size in range [0, 1]")
        exit(1)
    random.seed(42)

    dataset = CamVid(dataset_root, save_dir)
    
    try:
    	dataset.extract_frames()
    except Exception as e:
        pass

    dataset_entries = dataset.create_list()

    # make train/validation set separation
    dataset_entries = shuffle(dataset_entries)
    subset_boundary = int(len(dataset_entries) * train_set_size) + 1
    train_list = dataset_entries[:subset_boundary]
    val_list = dataset_entries[subset_boundary:]

    dataset.prepare_data(train_list, val_list, save_dir, max_image_size)