#!/usr/bin/env python

from neon.callbacks.callbacks import Callbacks
from neon.models import Model
from neon.optimizers import GradientDescentMomentum
from neon.util.argparser import NeonArgparser
from neon.transforms import Accuracy

import datasets
from datasets.imdb_wiki_face_dataset import IMDB_WIKI_FACE
from datasets.dummy import generate_data

import models
from models import get_weights_norm

from time import time
import os


def parse_args(docstring):
    from neon.util.argparser import NeonArgparser
    parser = NeonArgparser(docstring)
    parser.add_argument('--model_name', default=None)
    parser.add_argument('--data_root', default='./data')
    parser.add_argument('--restrict_dataset', default='0')
    return parser.parse_args()

def main(args)
    data_root = os.path.abspath(args.data_root)
    restrict_dataset = int(args.restrict_dataset)
    model_name = args.model_name

    dataset = IMDB_WIKI_FACE(data_root=data_root)
    train_set, val_set, lshape = dataset.gen_iterators_autoencoder()

    model_generator = None
    if (model_name is not None):
        model_generator = models.get_model_generator(model_name)
        if (model_generator is None):
            print("Not found model '%s'" % (model_name))
            exit(1)
        print("Using model '" + model_name + "'")
    else:
        model_generator = models.generate_mlp1b2_autoencoder_model
    model, cost = model_generator()
    # Load model with: --model_file path

    optimizer = GradientDescentMomentum(
        0.01, momentum_coef=0.9, stochastic_round=args.rounding, wdecay=0.0005)

    callbacks = Callbacks(model, eval_set=val_set, **args.callback_args)
    fit_time = time()
    model.fit(train_set, optimizer=optimizer,
        num_epochs=args.epochs, cost=cost, callbacks=callbacks)
    fit_time = time() - fit_time
    print('Learning time: %ds' % (fit_time))
    inference_time = time()
    accuracy = model.eval(val_set, metric=Accuracy())
    inference_time = time() - inference_time
    print('Accuracy = %.1f%%' % (accuracy * 100))
    print('Inference time: %ds' % (inference_time))

    # Save training state of the model: --serialize iterations_step --save_path path

    print("Train norm:", get_weights_norm(model))

if (__name__ == '__main__'):
    args = parse_args(__doc__)
    main(args)