import os
import h5py as h5
import numpy as np
from PIL import Image
from contextlib import closing


class BasicDataset:
    default_train_set_file_name = 'train'
    default_val_set_file_name = 'val'

    def __init__(self, dataset_root = '.', data_root = '.',
            train_set_file = default_train_set_file_name,
            val_set_file = default_val_set_file_name):
        self.dataset_root = os.path.abspath(dataset_root)
        self.data_root = os.path.abspath(data_root)
        self.train_set_file_name = train_set_file
        self.val_set_file_name = val_set_file

    def _load_subset(self, subset_data, lshape, subset_max_size=0):
        raise NotImplemented()

    def load(self, train_set_max_size=0, val_set_max_size=0):
        train_set_path = \
            self.data_root + '/' + self.train_set_file_name + '.h5'
        val_set_path = \
            self.data_root + '/' + self.val_set_file_name + '.h5'

        with closing(h5.File(train_set_path, 'r')) as train_set:
            lshape = tuple([int(e) for e in train_set['input'].attrs['lshape']])
            print("Loading train set... ", end='\r')
            X_train, y_train = self._load_subset(train_set, lshape, train_set_max_size)

        with closing(h5.File(val_set_path, 'r')) as val_set:
            print("Loading validation set... ", end='\r')
            X_val, y_val = self._load_subset(val_set, lshape, val_set_max_size)

        return (X_train, y_train, X_val, y_val, lshape)

    def _make_center_crop(self, image, crop_size):
        return image.crop((
            (image.size[0] - crop_size[0]) / 2,
            (image.size[1] - crop_size[1]) / 2,
            (image.size[0] + crop_size[0]) / 2,
            (image.size[1] + crop_size[1]) / 2,
            )) # center crop / padding to target size
