from neon.initializers import Gaussian, Kaiming, Constant, Xavier, GlorotUniform
from neon.layers import GeneralizedCost, Affine, Conv, Deconv, Pooling, BatchNorm, Activation, Dropout, DataTransform, Linear, Bias, MergeSum, SkipNode
from neon.transforms import Rectlin, Logistic, Softmax, PixelwiseSoftmax, Tanh, CrossEntropyBinary, CrossEntropyMulti, SumSquared, Normalizer, Explin
from neon.models import Model
from functools import reduce


def generate_mlp2b2_128_ae_stacked_step1_model(input_shape):
    output_size = reduce(lambda p, x: p * int(x), input_shape, 1)

    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh(), name='fc_1'),
        Affine(nout=output_size, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh(), name='fc_-1')
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=SumSquared())

    return (model, cost)

def generate_mlp2b2_128_ae_stacked_step1_encoder_model(input_shape):
    output_size = reduce(lambda p, x: p * int(x), input_shape, 1)

    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh(), name='fc_1'),
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=SumSquared())

    return (model, cost)

def generate_mlp2b2_128_ae_stacked_step2_model(input_shape):
    output_size = reduce(lambda p, x: p * int(x), input_shape, 1)

    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Xavier(), bias=Constant(0),
            activation=Tanh(), name='fc_1'),
        Affine(nout=64, init=Xavier(), bias=Constant(0),
            activation=Tanh(), name='fc_2'),
        Affine(nout=128, init=Xavier(), bias=Constant(0),
            activation=Tanh(), name='fc_-2')
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=SumSquared())

    return (model, cost)

def generate_mlp2b2_128_ae_stacked_step2_encoder_model(input_shape):
    output_size = reduce(lambda p, x: p * int(x), input_shape, 1)

    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Xavier(), bias=Constant(0),
            activation=Tanh(), name='fc_1'),
        Affine(nout=64, init=Xavier(), bias=Constant(0),
            activation=Tanh(), name='fc_2'),
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=SumSquared())

    return (model, cost)

def generate_mlp2b2_128_ae_stacked_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Xavier(), bias=Constant(0),
            activation=Tanh(), name='fc_1'),
        Affine(nout=64, init=Xavier(), bias=Constant(0),
            activation=Tanh(), name='fc_2'),
        Affine(nout=2, init=Xavier(), bias=Constant(0),
            activation=Logistic(shortcut=True), name='cls')
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2b2_128_ae_stacked_steps_model():
    steps = [
        'mlp2b2_128_ae_stacked_step1',
        'mlp2b2_128_ae_stacked_step2'
    ]
    return steps
