from neon.initializers import Gaussian, Kaiming, Constant, Xavier, GlorotUniform
from neon.layers import GeneralizedCost, Affine, Conv, Deconv, Pooling, BatchNorm, Activation, Dropout, DataTransform, Linear, Bias, MergeSum, SkipNode
from neon.transforms import Rectlin, Logistic, Softmax, PixelwiseSoftmax, Tanh, CrossEntropyBinary, CrossEntropyMulti, SumSquared, Normalizer, Explin
from neon.models import Model


def generate_mlp1_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1b_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1b2_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1_128_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1b_128_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1b2_128_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1_64_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=64, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1b_64_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=64, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1b2_64_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=64, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp1_ext_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=256, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2_2_32_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2_2_64_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=64, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=64, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2_128_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=64, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2b_128_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=64, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2b2_128_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=64, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2b3_128_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=128, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=64, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2_64_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=64, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2_ext_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=256, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=128, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2b_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2b2_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=32, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2bn_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1),
            activation=Tanh()),
        BatchNorm(),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        BatchNorm(),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2bnb2_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        BatchNorm(),
        Affine(nout=32, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        BatchNorm(),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2bnb_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1),
            activation=Tanh()),
        BatchNorm(),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        BatchNorm(),
        Affine(nout=2, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2bnb3_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1), bias=Constant(0),
            activation=Tanh()),
        BatchNorm(),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        BatchNorm(),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp2do_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Dropout(),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Dropout(),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_mlp3_model():
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=16, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=32, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=64, init=Gaussian(scale=0.1),
            activation=Tanh()),
        Affine(nout=2, init=Gaussian(scale=0.1),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)

def generate_tygrikon_mlp_model():
    # Adopted MLP model of https://github.com/Tygrikon/deep_learning_labs
    # learning parameters: lr=0.01, momentum_coef=0.9, wdecay=0.0005
    # -z 100 (batch size) -e 10 (epochs)
    # yields  about to 75% mean accuracy
    layers = [
        DataTransform(transform=Normalizer(divisor=128.0)),

        Affine(nout=400, init=Xavier(), bias=Constant(0),
            activation=Rectlin()),
        Affine(nout=50, init=Xavier(), bias=Constant(0),
            activation=Logistic()),
        Affine(nout=2, init=Xavier(), bias=Constant(0),
            activation=Logistic(shortcut=True))
    ]

    model = Model(layers=layers)

    cost = GeneralizedCost(costfunc=CrossEntropyBinary())

    return (model, cost)